/*
** nester - NES emulator
** Copyright (C) 2000  Darren Ranalli
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**
** This program is distributed in the hope that it will be useful, 
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
** Library General Public License for more details.  To obtain a 
** copy of the GNU Library General Public License, write to the Free 
** Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
**
** Any permitted reproduction of these routines, in whole or in part,
** must bear this legend.
*/

#include "stdlib.h"
#include "dc_sound_mgr.h"

static volatile unsigned long *position = (volatile unsigned long*)(0x10014 + 0xa0800000);

#define SAMPLE_BITS 16
#define NUM_SAMPLES 8
#define SAMPLE_RATE 44100
#define FPS         60
#define SOUND_BYTES 5880 // 735 * 8 (4 16bit sound frames)


dc_sound_mgr::dc_sound_mgr() : sound_mgr(SAMPLE_RATE, SAMPLE_BITS, SOUND_BYTES)
{
  buffer = (uint8 *)malloc(SOUND_BYTES);
  address = 0;
  current_address = 0x11000;
  last_pos = 0;
  last_write = 0;
  available_length = SOUND_BYTES;
  buffer_locked = false;
  *position = 0;
}


dc_sound_mgr::~dc_sound_mgr()
{
  free(buffer);
}


boolean 
dc_sound_mgr::lock(sound_buf_pos which, void** buf, uint32* buf_len)
{
  uint32 diff;
  uint32 current_pos = *position * 2;
  
  if (buffer_locked) return false;
  
  if (current_pos < this->last_pos) {
    diff = (SOUND_BYTES - this->last_pos + current_pos);
  } else {
    diff = (current_pos - this->last_pos);
  }
  
  if (diff == 0) return false;
  
  buffer_locked = true;
  this->available_length = diff;
  *buf_len = diff * (SAMPLE_BITS / 8) / 2;
  *buf = this->buffer; 
  return true;
}  
  

void dc_sound_mgr::unlock()
{
  uint32 len = this->available_length;
  
  if (!buffer_locked) return;
  buffer_locked = false;
  
  if ((this->last_pos + len) > SOUND_BYTES) {
    uint32 end_amount = SOUND_BYTES - this->last_pos;
    uint32 begin_amount = len - end_amount;
    snd_load (this->buffer, this->current_address + this->last_pos, end_amount);
    snd_load (this->buffer + end_amount, this->current_address, begin_amount);
    this->last_pos = begin_amount;
  } else {
    snd_load (this->buffer, (this->current_address + this->last_pos), len);
    this->last_pos = this->last_pos + len;
  }
}


/* return the alternation of SOUND_BUF_LOW and SOUND_BUF_HIGH,
   because of DoFrame. */
sound_mgr::sound_buf_pos
dc_sound_mgr::get_currently_playing_half()
{
  static sound_buf_pos cur_ph = SOUND_BUF_LOW;
  if (cur_ph == SOUND_BUF_LOW)
    cur_ph = SOUND_BUF_HIGH;
  else
    cur_ph = SOUND_BUF_LOW;
  return cur_ph;
}


